@@ -62,6 +62,7 @@ uncomment(char *buf)
 	bool incomment = false;
 	bool inesc = false;
 	bool inquote = false;
+	char quote = '\0';
 
 	for (s = t = buf; *s; s++) {
 		if (incomment) {
@@ -77,8 +78,12 @@ uncomment(char *buf)
 					inesc = false;
 				} else if (s[0] == '\\') {
 					inesc = true;
-				} else if (s[0] == '"') {
-					inquote = !inquote;
+				} else if (!inquote &&
+					   (s[0] == '"' || s[0] == '\'')) {
+					inquote = true;
+					quote = s[0];
+				} else if (inquote && s[0] == quote) {
+					inquote = false;
 				}
 
 				if (t != s) {
@@ -184,7 +189,11 @@ d_if(struct place *p, struct place *p2, char *line)
 	/* trim to fit, so the malloc debugging won't complain */
 	expr = dorealloc(expr, oldlen + 1, strlen(expr) + 1);
 
-	val = eval(&p3, expr);
+	if (ifstate->curtrue) {
+		val = eval(&p3, expr);
+	} else {
+		val = 0;
+	}
 	ifstate_push(p, val);
 	dostrfree(expr);
 }
