@@ -127,6 +127,7 @@ findeol(const char *buf, size_t start, size_t limit)
 	size_t i;
 	int incomment = 0;
 	bool inquote = false;
+	char quote = '\0';
 
 	for (i=start; i<limit; i++) {
 		if (incomment) {
@@ -139,10 +140,13 @@ findeol(const char *buf, size_t start, size_t limit)
 			i++;
 			incomment = 1;
 		} else if (i+1 < limit &&
-			   buf[i] == '\\' && buf[i+1] == '"') {
+			   buf[i] == '\\' && buf[i+1] != '\n') {
 			i++;
-		} else if (buf[i] == '"') {
-			inquote = !inquote;
+		} else if (!inquote && (buf[i] == '"' || buf[i] == '\'')) {
+			inquote = true;
+			quote = buf[i];
+		} else if (inquote && buf[i] == quote) {
+			inquote = false;
 		} else if (buf[i] == '\n') {
 			return i;
 		}
