@@ -108,6 +108,16 @@ static
 void
 commandline_macros_cleanup(void)
 {
+	unsigned i, num;
+	struct commandline_macro *cm;
+
+	num = array_num(&commandline_macros);
+	for (i=0; i<num; i++) {
+		cm = array_get(&commandline_macros, i);
+		dofree(cm, sizeof(*cm));
+	}
+	array_setsize(&commandline_macros, 0);
+	
 	array_cleanup(&commandline_macros);
 }
 
@@ -252,6 +262,18 @@ static
 void
 commandline_files_cleanup(void)
 {
+	unsigned i, num;
+	struct commandline_file *cf;
+
+	num = array_num(&commandline_files);
+	for (i=0; i<num; i++) {
+		cf = array_get(&commandline_files, i);
+		if (cf != NULL) {
+			dofree(cf, sizeof(*cf));
+		}
+	}
+	array_setsize(&commandline_files, 0);
+
 	array_cleanup(&commandline_files);
 }
 
@@ -293,6 +315,7 @@ read_commandline_files(void)
 	num = array_num(&commandline_files);
 	for (i=0; i<num; i++) {
 		cf = array_get(&commandline_files, i);
+		array_set(&commandline_files, i, NULL);
 		if (cf->suppress_output) {
 			save = mode.do_output;
 			mode.do_output = false;
@@ -886,13 +909,20 @@ check_arg_option(const char *opt, const struct place *argplace, char *arg)
 
 DEAD static
 void
-usage(const char *argv0)
+usage(const char *argv0, const char *fmt, ...)
 {
 	const char *progname;
+	va_list ap;
 
 	progname = strrchr(argv0, '/');
 	progname = progname == NULL ? argv0 : progname + 1;
 
+	fprintf(stderr, "%s: ", progname);
+	va_start(ap, fmt);
+	vfprintf(stderr, fmt, ap);
+	va_end(ap);
+	fprintf(stderr, "\n");
+
 	fprintf(stderr, "Usage: %s [options] [infile [outfile]]\n", progname);
 	fprintf(stderr, "Common options:\n");
 	fprintf(stderr, "   -C               Retain comments\n");
@@ -1000,7 +1030,7 @@ main(int argc, char *argv[])
 			i++;
 			continue;
 		}
-		usage(argv[0]);
+		usage(argv[0], "Invalid option %s", argv[i]);
 	}
 	if (i < argc) {
 		inputfile = argv[i++];
@@ -1009,7 +1039,7 @@ main(int argc, char *argv[])
 		outputfile = argv[i++];
 	}
 	if (i < argc) {
-		usage(argv[0]);
+		usage(argv[0], "Extra non-option argument %s", argv[i]);
 	}
 
 	mode.output_file = outputfile;
